// This file is part of Notepad++ project
// Copyright (C)2016 Don HO <don.h@free.fr>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// Note that the GPL places important restrictions on "derived works", yet
// it does not provide a detailed definition of that term.  To avoid      
// misunderstandings, we consider an application to constitute a          
// "derivative work" for the purpose of this license if it does any of the
// following:                                                             
// 1. Integrates source code from Notepad++.
// 2. Integrates/includes/aggregates Notepad++ into a proprietary executable
//    installer, such as those produced by InstallShield.
// 3. Links to a library or executes a program that does any of the above.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#include "asciiListView.h"
#include "Parameters.h"

#ifdef MOD_CHAR_PANEL_ENCODING
void AsciiListView::changeCodepage(int codepage)
{
	//if (_codepage == codepage)
	//	return;

	_codepage = codepage;
	for (int i = 128; i < 256; ++i)
	{
		generic_string s = getAscii((unsigned char)i);
		ListView_SetItemText(_hSelf, i, 3, (LPTSTR)s.c_str());
	}
}
#else
void AsciiListView::resetValues(int codepage)
{
	if (codepage == -1)
		codepage = 0;

	if (_codepage == codepage)
		return;

	ListView_DeleteAllItems(_hSelf);
	setValues(codepage);
}
#endif

generic_string AsciiListView::getAscii(unsigned char value)
{
	switch (value)
	{
		case 0:
			return TEXT("NULL");
		case 1:
			return TEXT("SOH");
		case 2:
			return TEXT("STX");
		case 3:
			return TEXT("ETX");
		case 4:
			return TEXT("EOT");
		case 5:
			return TEXT("ENQ");
		case 6:
			return TEXT("ACK");
		case 7:
			return TEXT("BEL");
		case 8:
			return TEXT("BS");
		case 9:
			return TEXT("TAB");
		case 10:
			return TEXT("LF");
		case 11:
			return TEXT("VT");
		case 12:
			return TEXT("FF");
		case 13:
			return TEXT("CR");
		case 14:
			return TEXT("SO");
		case 15:
			return TEXT("SI");
		case 16:
			return TEXT("DLE");
		case 17:
			return TEXT("DC1");
		case 18:
			return TEXT("DC2");
		case 19:
			return TEXT("DC3");
		case 20:
			return TEXT("DC4");
		case 21:
			return TEXT("NAK");
		case 22:
			return TEXT("SYN");
		case 23:
			return TEXT("ETB");
		case 24:
			return TEXT("CAN");
		case 25:
			return TEXT("EM");
		case 26:
			return TEXT("SUB");
		case 27:
			return TEXT("ESC");
		case 28:
			return TEXT("FS");
		case 29:
			return TEXT("GS");
		case 30:
			return TEXT("RS");
		case 31:
			return TEXT("US");
		case 32:
			return TEXT("Space");
		case 127:
			return TEXT("DEL");
		default:
		{
			TCHAR charStr[10];
			char ascii[2];
			ascii[0] = value;
			ascii[1] = '\0';
#ifdef FIX_CHAR_PANEL_CODEPAGE
			if (MultiByteToWideChar(_codepage, MB_ERR_INVALID_CHARS, ascii, -1, charStr, _countof(charStr)) > 0)
			{
				const static char *repsC1[] = {
					"PAD", "HOP", "BPH", "NBH", "IND", "NEL", "SSA", "ESA",
					"HTS", "HTJ", "VTS", "PLD", "PLU", "RI", "SS2", "SS3",
					"DCS", "PU1", "PU2", "STS", "CCH", "MW", "SPA", "EPA",
					"SOS", "SGCI", "SCI", "CSI", "ST", "OSC", "PM", "APC",
					"NBSP",
				};
				WCHAR u = charStr[0];
				if (0x0080 <= u && u <= 0x00A0)
					return WcharMbcsConvertor::getInstance()->char2wchar(repsC1[u - 0x0080], 65001);
				else
					return charStr;
			}
			else
				return L"";
#else
			MultiByteToWideChar(_codepage, 0, ascii, -1, charStr, sizeof(charStr));
			return charStr;
#endif
		}

	}
}

void AsciiListView::setValues(int codepage)
{
	_codepage = codepage;

	for (int i = 0 ; i < 256 ; ++i)
	{
		TCHAR dec[8];
		TCHAR oct[8];	//+[JOJO]
		TCHAR hex[8];
		TCHAR ctr[8];	//+[JOJO]
		generic_sprintf(dec, TEXT("%3d"), i);
		generic_sprintf(oct, TEXT("%03o"), i);	//+[JOJO]
		generic_sprintf(hex, TEXT("%02X"), i);
		generic_string s = getAscii(static_cast<unsigned char>(i));

		WCHAR *p = ctr; *p = L'\0';
		if (i < 0x20) {
			WCHAR *q;
			switch (i) {
			case 0x07: q = L"\\a"; break;
			case 0x08: q = L"\\b"; break;
			case 0x09: q = L"\\t"; break;
			case 0x0A: q = L"\\n"; break;
			case 0x0B: q = L"\\v"; break;
			case 0x0C: q = L"\\f"; break;
			case 0x0D: q = L"\\r"; break;
			case 0x1B: q = L"\\e"; break;
			case 0x20: q = L"\\s"; break;
			default:   q = L"";
			}
			p += swprintf(p, L"^%c %s", 0x40 + i, q);
		}

		std::vector<generic_string> values2Add;

		values2Add.push_back(dec);
		values2Add.push_back(oct);	//+[JOJO]
		values2Add.push_back(hex);
		values2Add.push_back(s);
		values2Add.push_back(ctr);	//+[JOJO]

		addLine(values2Add);
	}
}
